/***************************************************************************
 *   Copyright (C) 2010 by Stephane List                                   *
 *   slist@lilotux.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QMessageBox>
#include <QDebug>
#include <QUrl>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QTime>
#include <QTimer>

#include "ui_mainwindow.h"
#include "ui_open.h"

#include "mainwindow.h"
#include "version.h"

MainWindow::MainWindow(QWidget *parent) :
        QMainWindow(parent),
        ui(new Ui::MainWindow),
        file_(""),
        reload_(true)

{
    ui->setupUi(this);

    connect(ui->action_FS, SIGNAL(triggered()), this, SLOT(showFullScreen()));
    connect(ui->action_Max, SIGNAL(triggered()), this, SLOT(showMaximized()));
    connect(ui->action_Normal, SIGNAL(triggered()), this, SLOT(showNormal()));
    connect(ui->actionClose, SIGNAL(triggered()), this, SLOT(close()));
    connect(ui->action_QT, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

    QStringList sl;
    sl = QCoreApplication::arguments ();
    qDebug() << sl;

    if (sl.contains("-fs", Qt::CaseInsensitive))

    {
        showMaximized(); // Maximized before prevents from having the window position in normal mode out of control
        showFullScreen();

        // don't know how to gain focus in full screen ?? Maybe it's a bug in gnome ?
        //        activateWindow();
        //        raise();

    }

    if (sl.contains("-max", Qt::CaseInsensitive))
        showMaximized();

    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(update_status()));
    timer->start(1000);

    if (sl.size() < 2)
    {
        qDebug("Nothing to display");
        return;
    }

    int t = sl.indexOf("-t");
    if (t>0 && (t < sl.size()))
    {
        setWindowTitle(sl[t+1]);
        qDebug() << "title:" << sl[t+1];
    }

    file_ = sl[sl.size() - 1];
    load_image();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::reload_image()
{
    if (reload_)
        load_image();
}

void MainWindow::load_image()
{
    if (!file_.isEmpty())
    {
        ui->statusBar->showMessage("Load file : " + file_);
        if (file_.startsWith("http://"))
        {
            get_image(file_);
        }
        else
        {
            if (file_.startsWith("file:/"))
            {
                file_.remove(0,6);
            }
            QPixmap pix(file_);
            display_image(pix);
        }
    }
}

void MainWindow::on_action_A_propos_triggered()
{
    QMessageBox::about(this,
                       tr("About"),
                       tr("<p><b>qdisplay</b></p>"
                          "<p>Version qdisplay: %1"
                          "<p>Version %2: %3.%4"
			  "<p>Licence : GPL V3"
			  "<p>Author : Stephane List (slist@lilotux.net)"
                          "</p>")
                       .arg(VERSION)
#ifdef __GNUC__
                       .arg("GCC")
                       .arg(__GNUC__)
                       .arg(__GNUC_MINOR__)
#elif defined(_MSC_VER)
                       .arg("VC++")
                       .arg(_MSC_VER / 100)
                       .arg(_MSC_VER % 100,2,10,QChar('0'))
#else
                       .arg("Compiler")
                       .arg("?")
                       .arg("?")
#endif
                       );
}

void MainWindow::display_image(const QPixmap & pix)
{
    if (pix.isNull())
    {
        ui->toto->setPixmap(QPixmap(QString::fromUtf8(":/file_broken.png")));
    }
    else
    {
        ui->toto->setPixmap(pix);
    }
}

void MainWindow::get_image(const QString & s)
{
    QNetworkAccessManager *manager = new QNetworkAccessManager(this);
    connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(replyFinished(QNetworkReply*)));
    manager->get(QNetworkRequest(QUrl(s,QUrl::TolerantMode)));
    qDebug()<< "url submited :        " << s;
}

void MainWindow::replyFinished(QNetworkReply * reply)
{
    QByteArray ba;
    ba = reply->readAll();
    reply->manager()->deleteLater();
    reply->deleteLater();

    qDebug()<< "url corrected: " << reply->url();

    QPixmap p;
    p.loadFromData(ba);
    display_image(p);
}

// time in second for the website to refresh graphics
#define OFFSET 1

void MainWindow::update_status()
{

    QTime t;
    t = QTime::currentTime();

    if (reload_)
    {
        if ((t.second())%15 == OFFSET)
        {
            load_image();
        }
        else
        {
            ui->statusBar->showMessage(t.toString("HH:mm:ss"));
        }
    }
    else
        ui->statusBar->showMessage(t.toString("HH:mm:ss") + tr(" (reload : off)"));
}

void MainWindow::on_actionRechargement_toggled(bool b)
{
    reload_ = b;
    if (b)
    {
        qDebug("b is on");
        load_image();
    }
    else
        qDebug("b is off");
}

void MainWindow::on_action_Ouvrir_triggered()
{
    qDebug() << "Open" << file_;
    Ui::OpenDialog o;
    QDialog d;
    o.setupUi(&d);
    o.url->setText(file_);
    if (d.exec() ==QDialog::Accepted)
    {
        file_ = o.url->text();
        load_image();
    }
}
